@@ -19,9 +19,11 @@ from flexai.capability import Capability
 from flexai.llm import Client, DefaultClient
 from flexai.message import (
     AIMessage,
+    DataBlock,
     Message,
     MessageContent,
     SystemMessage,
+    TextBlock,
     ThoughtBlock,
     ToolCall,
     ToolResult,
@@ -158,7 +160,7 @@ class Agent:
             Intermediate message chunks followed by the modified AI response.
 
         Raises:
-            ValueError: If modified response was not yielded from a capability.
+            ValueError: If modified response was not yielded from capability.
         """
         # Iterate through the capabilities and modify the response.
         for capability in self.capabilities:
@@ -195,6 +197,7 @@ class Agent:
 
         Yields:
             Intermediate message chunks followed by the final tool result.
+
         """
         tool = self.toolbox[tool_call.name]
         is_error = False
@@ -238,6 +241,11 @@ class Agent:
 
         end_time = time.time()
 
+        if isinstance(result, dict):
+            result = DataBlock(data=result)
+        elif not isinstance(result, MessageContent):
+            result = TextBlock(text=str(result))
+
         yield ToolResult(
             tool_call_id=tool_call.id,
             result=result,
