@@ -333,13 +333,21 @@ class AnthropicClient(Client):
         if isinstance(system, str):
             system = SystemMessage([TextBlock(system)])
 
+        thinking_args = (
+            {"thinking": {"type": "disabled"}}
+            if thinking_budget == 0
+            else {"thinking": {"type": "enabled", "budget_tokens": thinking_budget}}
+            if thinking_budget and thinking_budget > 0
+            else {}
+        )
+
         kwargs = {
             "max_tokens": self.max_tokens,
             "messages": self._format_content(messages, allow_tool),
             "model": self.model,
             "temperature": temperature,
             "extra_headers": self.extra_headers,
-            **({"thinking": {"type": "disabled"}} if (thinking_budget == 0) else {}),
+            **thinking_args,
         }
 
         # Only add system message if it's not empty
