@@ -1,3 +1,4 @@
+import base64
 import functools
 import json
 import operator
@@ -332,6 +333,16 @@ class GeminiClient(Client):
         if part_object.function_call is not None:
             yield get_tool_call(part_object.function_call)
 
+        if (
+            part_object.inline_data is not None
+            and part_object.inline_data.data is not None
+            and part_object.inline_data.mime_type is not None
+        ):
+            yield ImageBlock(
+                image=part_object.inline_data.data,
+                mime_type=part_object.inline_data.mime_type,
+            )
+
     @classmethod
     def _format_message_content(
         cls,
@@ -349,7 +360,6 @@ class GeminiClient(Client):
 
         Raises:
             ValueError: If unsupported content type or tool call context issues.
-            TypeError: If tool result format is invalid.
         """
         if isinstance(content, str):
             return [{"text": content}]
@@ -370,7 +380,9 @@ class GeminiClient(Client):
             return {
                 "inlineData": {
                     "mimeType": content.mime_type,
-                    "data": content.image,
+                    "data": base64.b64encode(content.image).decode("utf-8")
+                    if isinstance(content.image, bytes)
+                    else content.image,
                 }
             }
         if isinstance(content, TextBlock):
@@ -393,14 +405,18 @@ class GeminiClient(Client):
             }
         if isinstance(content, ToolResult):
             formatted_result = content.result
+            if isinstance(formatted_result, DataBlock):
+                formatted_result = TextBlock(text=json.dumps(formatted_result.data))
+
             if isinstance(formatted_result, str):
                 formatted_result = {
                     "result": formatted_result,
                 }
-            if not isinstance(formatted_result, dict):
-                raise TypeError(
-                    f"Expected tool reuslt to be of type str or dict, instead got {type(formatted_result)}"
+            elif not isinstance(formatted_result, dict):
+                formatted_result = cls._format_message_content(
+                    formatted_result, name_context=name_context
                 )
+
             if content.tool_call_id not in name_context:
                 raise ValueError(
                     f"Tool call {content.tool_call_id} not found in context, but a result for it was found."
@@ -425,6 +441,8 @@ class GeminiClient(Client):
         use_url_context: bool = False,
         use_google_search: bool = False,
         google_search_dynamic_threshold: float | None = None,
+        allow_text=True,
+        allow_image=False,
         **kwargs,
     ):
         """Build parameters for Gemini API request.
@@ -439,6 +457,8 @@ class GeminiClient(Client):
             use_url_context: Whether to enable URL context retrieval.
             use_google_search: Whether to enable Google Search integration.
             google_search_dynamic_threshold: Threshold for dynamic search retrieval.
+            allow_text: Whether to allow text content in responses.
+            allow_image: Whether to allow image content in responses.
             kwargs: Additional parameters including structured output models.
 
         Returns:
@@ -542,6 +562,16 @@ class GeminiClient(Client):
                 }
             )
 
+        response_modalities = []
+
+        if allow_text:
+            response_modalities.append(types.Modality.TEXT)
+
+        if allow_image:
+            response_modalities.append(types.Modality.IMAGE)
+
+        config_args["response_modalities"] = response_modalities
+
         config = types.GenerateContentConfig(
             **config_args,
         )
@@ -563,6 +593,8 @@ class GeminiClient(Client):
         use_url_context: bool = False,
         use_google_search: bool = False,
         google_search_dynamic_threshold: float | None = None,
+        allow_text: bool = True,
+        allow_image: bool = False,
         **kwargs,
     ) -> AIMessage:
         """Get a chat response from the Gemini model.
@@ -577,6 +609,8 @@ class GeminiClient(Client):
             use_url_context: Whether to enable URL context retrieval.
             use_google_search: Whether to enable Google Search integration.
             google_search_dynamic_threshold: Threshold for dynamic search retrieval.
+            allow_text: Whether to allow text content in responses.
+            allow_image: Whether to allow image content in responses.
             kwargs: Additional parameters including structured output models.
 
         Returns:
@@ -606,6 +640,8 @@ class GeminiClient(Client):
             use_url_context=use_url_context,
             use_google_search=use_google_search,
             google_search_dynamic_threshold=google_search_dynamic_threshold,
+            allow_text=allow_text,
+            allow_image=allow_image,
             **extra_param_args,
         )
         start = time.time()
@@ -637,7 +673,12 @@ class GeminiClient(Client):
                 message="Gemini did not respond with any content.", provider="gemini"
             )
         formatted_content_parts: list[
-            TextBlock | ThoughtBlock | ToolCall | URLContextBlock | GroundingBlock
+            ImageBlock
+            | TextBlock
+            | ThoughtBlock
+            | ToolCall
+            | URLContextBlock
+            | GroundingBlock
         ] = []
 
         for part in response_content_parts:
