@@ -282,7 +282,7 @@ class OpenAIClient(Client):
     @classmethod
     def _format_message_content(
         cls,
-        contents: Sequence[ImageBlock | TextBlock | ToolCall | ToolResult],
+        contents: Sequence[MessageContent],
     ) -> list[dict[str, Any]]:
         return [
             {"type": "text", "text": content.text}
@@ -306,8 +306,16 @@ class OpenAIClient(Client):
             else {
                 "role": "tool",
                 "tool_call_id": content.tool_call_id,
-                "content": json.dumps(content.result),
+                "content": cls._format_message_content([content.result]),
             }
+            if isinstance(content, ToolResult)
+            else (
+                (_ for _ in ()).throw(
+                    TypeError(
+                        f"Tried to send {content} to openai, which is of an unsupported type."
+                    )
+                )
+            )
             for content in contents
         ]
 
